==> rheolef_6.1.orig.tar.gz: rheolef-6.1/skit/ptst2/solver_ic0_tst.cc <==
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///
/// =========================================================================
// test of the solver
//
#include "rheolef/skit.h"
using namespace rheolef;
using namespace std;
int main (int argc, char **argv) { 
  environment rheolef (argc, argv);
  communicator comm;
  double err_tol = 1e-7;

  // read the matrix
  csr<Float> a;
  din >> a;
  a.set_symmetry(true);

  // convert csr a to csc trans(a) pastix format (with a transpose)
  solver_option_type opt;
  opt.verbose_level = 5;
  solver sa = ic0(a, opt); 
  // set x(dis_i) = dis_i
  vec<double> x_ex (a.row_ownership());
  for (size_t i = 0; i < x_ex .size(); i++) {
    x_ex [i] = i + x_ex .ownership().first_index();
  }
  // compute the rhs and solve
  vec<double> b = a*x_ex ;
  vec<double> x (b.ownership(), 0.);
  size_t max_iter = 10000;
  Float tol = 1e-15;
  int status = pcg (a, x, b, sa, max_iter, tol, &derr);

  // check the result
  double err = sqrt(dot (x-x_ex,x-x_ex));
  warning_macro ("error=|x-x_ex|=" << err);

  // build another matrix with the same sparse structure
  csr<double> a2 = a+a;
  sa.update_values (a2); 
  vec<double> b2 = a2*x_ex ;
  max_iter = 10000;
  tol = 1e-15;
  x = 0;
  status = pcg (a, x, b, sa, max_iter, tol, &derr);
  double err2 = sqrt(dot (x-x_ex,x-x_ex));
  warning_macro ("error2=|x-x_ex|=" << err2);
  
  return max(err,err2) < err_tol ? 0 : 1;
}

==> rheolef_6.7.orig.tar.gz: rheolef-6.7/skit/ptst2/solver_ic0_tst.cc <==
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///
/// =========================================================================
// test of the solver
//
#include "rheolef/skit.h"
using namespace rheolef;
using namespace std;
int main (int argc, char **argv) { 
  environment rheolef (argc, argv);
  communicator comm;
  Float err_tol = 1e-7;

  // read the matrix
  csr<Float> a;
  din >> a;
  a.set_symmetry(true);

  // convert csr a to csc trans(a) pastix format (with a transpose)
  solver_option_type opt;
  opt.verbose_level = 5;
  solver sa = ic0(a, opt); 
  // set x(dis_i) = dis_i
  vec<Float> x_ex (a.row_ownership());
  for (size_t i = 0; i < x_ex .size(); i++) {
    x_ex [i] = i + x_ex .ownership().first_index();
  }
  // compute the rhs and solve
  vec<Float> b = a*x_ex ;
  vec<Float> x (b.ownership(), 0.);
  size_t max_iter = 10000;
  Float tol = 1e-15;
  int status = pcg (a, x, b, sa, max_iter, tol, &derr);

  // check the result
  Float err = sqrt(dot (x-x_ex,x-x_ex));
  warning_macro ("error=|x-x_ex|=" << err);

  // build another matrix with the same sparse structure
  csr<Float> a2 = a+a;
  sa.update_values (a2); 
  vec<Float> b2 = a2*x_ex ;
  max_iter = 10000;
  tol = 1e-15;
  x = 0;
  status = pcg (a, x, b, sa, max_iter, tol, &derr);
  Float err2 = sqrt(dot (x-x_ex,x-x_ex));
  warning_macro ("error2=|x-x_ex|=" << err2);
  
  return max(err,err2) < err_tol ? 0 : 1;
}
